@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#! @PYSHEBANG@
 # -*- coding: utf-8 -*-
 #
 # ntpq - query an NTP server using mode 6 commands
@@ -14,7 +14,6 @@ from __future__ import print_function, division
 
 import cmd
 import getopt
-import hashlib
 import os
 import re
 import resource
@@ -71,7 +70,7 @@ class Ntpq(cmd.Cmd):
         #  so I am leaving them, and possibly duplicating them.
         self.rawmode = False            # Flag which indicates raw mode output.
         self.directmode = False         # Flag for direct MRU output.
-        self.showhostnames = True       # If false, display numeric IPs
+        self.showhostnames = 1          # If & 1 false, display numeric IPs
         self.showunits = False          # If False, show old style float
         self.auth_delay = 20            # delay time (default 20msec)
         self.wideremote = False         # show wide remote names?
@@ -442,7 +441,7 @@ usage: timeout [ msec ]
                 value = queried[name][0]
                 rawvalue = queried[name][1]
                 if fmt in (NTP_ADD, NTP_ADP):
-                    if self.showhostnames:
+                    if self.showhostnames[0]:
                         if self.debug:
                             self.say("DNS lookup begins...")
                         value = ntp.util.canonicalize_dns(
@@ -563,20 +562,29 @@ usage: passwd []
         if not line:
             pass
         elif line == "yes":
-            self.showhostnames = True
+            self.showhostnames = 1
         elif line == "no":
-            self.showhostnames = False
+            self.showhostnames = 0
+        elif line == 'hostnum':
+            self.showhostnames = 2
+        elif line == 'hostname':
+            self.showhostnames = 3
         else:
             self.say("What?\n")
-        if self.showhostnames:
-            self.say("hostnames being shown\n")
+            pass
+        if self.showhostnames & 1:
+            self.say('resolved hostnames being shown\n')
         else:
-            self.say("hostnames not being shown\n")
+            self.say('resolved hostnames not being shown\n')
+        if self.showhostnames & 2:
+            self.say('supplied hostnames being shown\n')
+        else:
+            self.say('supplied hostnames not being shown\n')
 
     def help_hostnames(self):
         self.say("""\
 function: specify whether hostnames or net numbers are printed
-usage: hostnames [yes|no]
+usage: hostnames [yes|no|hostname|hostnum]
 """)
 
     def do_debug(self, line):
@@ -1207,6 +1215,42 @@ usage: config_from_file <configuration filename>
         for entry in entries:
             self.say(self.formatter.summary(entry) + "\n")
 
+    def do_noflake(self):
+        """Disables the dropping of control packets by ntpq for testing."""
+        self.session.flakey = False
+
+    def help_noflake(self):
+        """Print help for noflake."""
+        self.say("""\
+function: Disables the dropping of received packets by ntpq for testing.
+usage: noflake
+""")
+
+    def do_doflake(self, line):
+        """Drop some received packets for testing.
+
+        Probabilities 0 and 1 should be certainly accepted
+        and discarded respectively. No default, but 0.1
+        should be a one in ten loss rate.
+        """
+        try:
+            _ = float(line)
+            if not 0 < _ < 1:
+                raise ValueError
+            self.session.flakey = _
+        except ValueError:
+            self.say('Flakiness must be a (positive) float less than 1.')
+
+    def help_doflake(self):
+        """Print help for doflake."""
+        self.say("""\
+function: Enables the dropping of control packets by
+ntpq for testing. Probabilities 0 and 1 should be
+certainly accepted and discarded respectively. No
+default, but 0.1 should be a one in ten loss rate.
+usage: doflake <probability>
+""")
+
     def do_mrulist(self, line):
         """display the list of most recently seen source addresses,
            tags mincount=... resall=0x... resany=0x..."""
@@ -1610,12 +1654,12 @@ if __name__ == '__main__':
     try:
         (options, arguments) = getopt.getopt(
             sys.argv[1:],
-            "46a:c:dD:hk:npVwW:ul:",
+            "46a:c:dD:hk:npsSVwW:ul:",
             ["ipv4", "ipv6", "authentication=",
              "command=", "debug", "set-debug-level=",
              "help", "keyfile", "numeric", "peers",
              "version", "wide", "width=", "units",
-             "logfile="])
+             "logfile=", "srcname", "srcnumber"])
     except getopt.GetoptError as e:
         sys.stderr.write("%s\n" % e)
         sys.stderr.write(usage)
@@ -1650,11 +1694,15 @@ if __name__ == '__main__':
             sys.stderr.write(usage)
             raise SystemExit(0)
         elif switch in ("-n", "--numeric"):
-            interpreter.showhostnames = False
+            interpreter.showhostnames = 0
         elif switch in ("-p", "--peers"):
             interpreter.ccmds.append("peers")
         elif switch in ("-k", "--keyfile"):
             keyfile = val
+        elif switch in ("-s", "--srcname"):
+            interpreter.showhostnames = 3
+        elif switch in ("-S", "--srcnumber"):
+            interpreter.showhostnames = 2
         elif switch in ("-V", "--version"):
             sys.stdout.write("ntpq %s\n" % version)
             raise SystemExit(0)
