@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#! @PYSHEBANG@
 # -*- coding: utf-8 -*-
 """\
 ntpviz - visualizer for NTP log files
@@ -791,9 +791,9 @@ plot \\
 
         for key in gpslist:
             plot_template += """\
-'-' using 1:2 title '%(key)s TDOP' with line ls 1, \\
-'-' using 1:3 title '%(key)s nSat' with line ls 2 axis x1y2, \\
-""" % locals()
+'-' using 1:2 title '%s TDOP' with line ls 1, \\
+'-' using 1:3 title '%s nSat' with line ls 2 axis x1y2, \\
+""" % (key,)
 
         # strip the trailing ", \\n"
         plot_template = plot_template[:-4] + "\n"
@@ -1288,7 +1288,7 @@ plot \
 def local_offset_multiplot(statlist):
     "Plot comparative local offsets for a list of NTPViz objects."
 
-    out = []
+    out = {}
     out['size'] = args.img_size
     out['terminal'] = args.terminal
 
@@ -1612,7 +1612,10 @@ Python by ESR, concept and gnuplot code by Dan Drown.
         if 2 < args.debug_level:
             sys.stderr.write("ntpviz: INFO: now running at nice: %s\n" % nice)
 
-    for fontpath in ("/usr/share/fonts/liberation",
+    for fontpath in ("@PREFIX@/share/fonts/liberation",
+                     "@PREFIX@/share/fonts/liberation-fonts",
+                     "@PREFIX@/share/fonts/truetype/liberation",
+                     "/usr/share/fonts/liberation",
                      "/usr/share/fonts/liberation-fonts",
                      "/usr/share/fonts/truetype/liberation"):
 
