@@ -207,6 +207,7 @@ from __future__ import print_function, division
 import getpass
 import hmac
 import os
+import random
 import select
 import socket
 import string
@@ -740,6 +741,7 @@ class ControlSession:
         self.logfp = sys.stdout
         self.nonce_xmit = 0
         self.slots = 0
+        self.flakey = None
 
     def warndbg(self, text, threshold):
         ntp.util.dolog(self.logfp, text, self.debug, threshold)
@@ -1019,6 +1021,10 @@ class ControlSession:
                 # usually, errno 111: connection refused
                 raise ControlException(SERR_SOCKET)
 
+            if self.flakey and self.flakey >= random.random():
+                warndbg('Flaky: I deliberately dropped a packet.', 1)
+                rawdata = None
+
             warndbg("Received %d octets" % len(rawdata), 3)
             rpkt = ControlPacket(self)
             try:
@@ -1121,7 +1127,7 @@ class ControlSession:
                         warn("Response packet:\n%s\n" % repr(self.response))
                     elif self.debug >= 2:  # pragma: no cover
                         # FIXME: Garbage when retrieving assoc list (binary)
-                        eol = self.response.find("\n")
+                        eol = self.response.find(b"\n")
                         firstline = self.response[:eol]
                         warn("First line:\n%s\n" % repr(firstline))
                     return None
@@ -1328,7 +1334,6 @@ This combats source address spoofing
 
     def __mru_analyze(self, variables, span, direct):
         """Extracts data from the key/value list into a more useful form"""
-        curidx = -1
         mru = None
         nonce = None
         items = list(variables.items())
@@ -1598,6 +1603,11 @@ def parse_mru_variables(variables):
                  "maxlstint", "minlstint", "laddr", "recent",
                  "sort", "frags", "limit"):
             continue
+        elif k.startswith('addr.') or k.startswith('last.'):
+            kn = k.split('.')
+            if len(kn) != 2 or kn[1] not in map(str, list(range(16))):
+                raise ControlException(SERR_BADPARAM % k)
+            continue
         else:
             raise ControlException(SERR_BADPARAM % k)
     if 'frags' in variables:
@@ -1766,6 +1776,10 @@ class Authenticator:
                             ntp.poly.polybytes(passwd), keytype)
         if not mac2:
             return False
-        return hmac.compare_digest(mac, mac2)
+        # typically preferred to avoid timing attacks client-side (in theory)
+        try:
+            return hmac.compare_digest(mac, mac2) # supported 2.7.7+ and 3.3+
+        except AttributeError:
+            return mac == mac2  # solves issue #666
 
 # end
